//*******************************************************
//*****************************************
// Numitron(IV-9) Clock Program Mega328
// CPU Clock:8MHz
//   iq[Yf[^ UWlgɂj
//
// file name: disp_iv9.c
//
//    SVZOkdcɃX^eBbN\邪
//    VA]̂kdchCo[iskbTXPUjgB
//    PC2:LE   latch enable      0=Lighting
//    PC3:OE   output enable     0=out enable
//    PC4:CLK  serial pulse      0=Lighting
//    PC5:SDO  serial data       0=Lighting
//
//  2016/10/23  New(TAB4)  Taishi
//  2016/12/26  Modify     Taishi 邳ݒǉ
//*******************************************************

#include	<avr/io.h>

#include	"four_iv9_clock.h"
#include	"disp_tlc5916_iv9.h"

// \̖邳̒l
// 								D7;CC0, D6:CC1, D5:CC2, D4:CC3, D3:CC4, D2:CC5, D1:HC, D0:CM
#define	SEG7_BRIGHT_HIGH		0b11111111						// \Ԗ邭鎞
#define	SEG7_BRIGHT_MID			0b00001111						// \𒆂炢鎞
#define	SEG7_BRIGHT_LOW			0b00000111						// \ԈÂ鎞

static	unsigned char	seg7_bright = SEG7_BRIGHT_HIGH;			// \Ԓ̂neeɂ銄i邳ςj

static	unsigned char	seg7_data[4] ={0xff,0xff,0xff,0xff};	// \Vf[^i[0]ʁij

static	const unsigned char	hex_to_7seg[16]={
		// D7;dp, D6:g, D5:f, D4:e, D3:d, D2:c, D1:b, D0:a
		0b00111111,				// 0
		0b00000110,				// 1
		0b01011011,				// 2
		0b01001111,				// 3
		0b01100110,				// 4
		0b01101101,				// 5
		0b01111101,				// 6
		0b00000111,				// 7
		0b01111111,				// 8
		0b01101111,				// 9
		0b01110111,				// A
		0b01111100,				// B
		0b01011000,				// C
		0b01011110,				// D
		0b01111001,				// E
		0b01110001,				// F
};

// *****************************
//  VZOp̏ݒ
//  @ Ȃ
//
//  |[g(rbg)ݒ肷
//    VA]̂kdchCo[gB
//    PC2:LE   latch enable      0=Lighting
//    PC3:OE   output enable     0=out enable
//    PC4:CLK  serial pulse      0=Lighting
//    PC5:SDO  serial data       0=Lighting
// *****************************
void	init_7seg(void)
{
	PORTC |=  (1 << PC3);												// OE=High
	PORTC &= ~((1 << PC2) | (1 << PC4) | (1 << PC5) );					// ndȊO͑SĂk
	DDRC  |= ((1 << DDC2) | (1 << DDC3) | (1 << DDC4) | (1 << DDC5) );	// VA]̃rbgo͂ɂ
}
// *****************************
//  VZO\̗L̃Zbg
// *****************************
void	set_7seg_enable(void)
{
	PORTC &= ~(1 << PC3);			// OE=Low
}
// *****************************
//  VZO\̖̃Zbg
// *****************************
void	set_7seg_disable(void)
{
	PORTC |=  (1 << PC3);			// OE=High
}
// *****************************
//  VZOf[^̃Zbg
//  @
//@@@@FPUiiO`PTAPUȏ̓uNj
//@@@F̏ԁ@OAP_
//@@@@FwiO`RAOōj
//
// *****************************
void set_7seg_data(unsigned char h, unsigned char dp, unsigned char n)
{
	unsigned char	seg;

	if(16 <= h){
		seg = 0;						// uN
	}else{
		seg = hex_to_7seg[h & 0x0f];	// PUiVZOf[^ɕϊ
	}
	if(dp != 0){
		seg |= 0x80;					// ǉ
	}

	seg7_data[n & 0x03] = seg;			// \Vf[^ɓ

}
// *****************************
//  VZOf[^֕ϊS
//  @
//@@@@strFŜPUiiO`PTj̃|C^[
//@@@@ dpFŜỉEj̃|C^[
//
//@@ŜPUi͉SrbgL
// *****************************
void	chang_7seg_data(unsigned char *str,unsigned char *dp)
{
	unsigned char	i;

	for(i=0;i<4;i++){
		set_7seg_data(*(str+i), *(dp+i), i);		//  VZOf[^̃Zbg
	}
}
// *****************************
//  VZOf[^̂𒼐ڃZbg
//  @
//@@@F̏ԁ@OAP_
//@@@@FwiO`RAOj
//
// *****************************
void	set_7seg_dp(unsigned char dp, unsigned char n)
{
	if(dp != 0){
		seg7_data[n & 0x03] &= ~0x80;		// _
	}else{
		seg7_data[n & 0x03] |=  0x80;		// 
	}
}

// *****************************
//  VZO̕\
//    \sɂ̊֐Ă
//    VA]̂kdchCo[gB
//    PC2:LE   latch enable      0=Lighting
//    PC3:OE   output enable     0=out enable
//    PC4:CLK  serial pulse      0=Lighting
//    PC5:SDO  serial data       0=Lighting
// *****************************
void	disp_7seg(void)
{
	unsigned char	keta=0;
	unsigned char	b = 1;

	PORTC &= ~( (1 << PC2) | (1 << PC4));		// Latch disable, clock=Low

	for(keta = 0;keta < 4; keta++){
		for(b = 0x80; b != 0; b >>= 1){
			if((seg7_data[3 - keta] & b) == 0){		// out bit = Low
				PORTC &= ~(1 << PC5);
			}else{									// out bit = high
				PORTC |=  (1 << PC5);
			}
			nop();
			PORTC |=  (1 << PC4);					// clock=high
			nop();
			PORTC &=  ~(1 << PC4);					// clock=low
		}
	}

	nop();
	PORTC |= (1 << PC2);							// Latch enable
	nop();
	PORTC &= ~(1 << PC2);							// Latch disable
}

// *****************************
//  Special [hɃZbg
// *****************************
void	set_special_mode(void)
{
	PORTC &= ~(1 << PC2);			// LE=Low
	PORTC |=  (1 << PC3);			// OE=High
	nop();
	PORTC |=  (1 << PC4);			// clock=high (1)
	nop();
	PORTC &=  ~(1 << PC4);			// clock=low

	PORTC &= ~(1 << PC3);			// OE=Low
	nop();
	PORTC |=  (1 << PC4);			// clock=high (2)
	nop();
	PORTC &=  ~(1 << PC4);			// clock=low

	PORTC |=  (1 << PC3);			// OE=High
	nop();
	PORTC |=  (1 << PC4);			// clock=high (3)
	nop();
	PORTC &=  ~(1 << PC4);			// clock=low

	PORTC |= (1 << PC2);			// LE=High
	nop();
	PORTC |=  (1 << PC4);			// clock=high (4)
	nop();
	PORTC &=  ~(1 << PC4);			// clock=low

	PORTC &= ~(1 << PC2);			// LE=Low
	nop();
	PORTC |=  (1 << PC4);			// clock=high (5)
	nop();
	PORTC &=  ~(1 << PC4);			// clock=low
}
// *****************************
//  Normal [hɃZbg
// *****************************
void	set_normal_mode(void)
{
	PORTC &= ~(1 << PC2);			// LE=Low
	PORTC |=  (1 << PC3);			// OE=High

	nop();
	PORTC |=  (1 << PC4);			// clock=high (1)
	nop();
	PORTC &=  ~(1 << PC4);			// clock=low

	PORTC &= ~(1 << PC3);			// OE=Low
	nop();
	PORTC |=  (1 << PC4);			// clock=high (2)
	nop();
	PORTC &=  ~(1 << PC4);			// clock=low

	PORTC |=  (1 << PC3);			// OE=High
	nop();
	PORTC |=  (1 << PC4);			// clock=high (3)
	nop();
	PORTC &=  ~(1 << PC4);			// clock=low

	PORTC &= ~(1 << PC2);			// LE=Low
	nop();
	PORTC |=  (1 << PC4);			// clock=high (4)
	nop();
	PORTC &=  ~(1 << PC4);			// clock=low

	PORTC &= ~(1 << PC2);			// LE=Low
	nop();
	PORTC |=  (1 << PC4);			// clock=high (5)
	nop();
	PORTC &=  ~(1 << PC4);			// clock=low
}
// *****************************
//  Configuration Code ̏
//    dl
//     n:0--255(D0:CM,D1:HC,D2--D7:CC0--CC5)
//    PC2:LE   latch enable
//    PC3:OE*  output enable
//    PC4:CLK  serial clock
//    PC5:SDO  serial data(TLC5916-SDI)
// *****************************
void	write_config(unsigned char n)
{
	unsigned char	keta=0;
	unsigned char	b = 1;

	PORTC |=  (1 << PC3);					// OE*:High
	PORTC &= ~(1 << PC2);					// LE :Low
	PORTC &= ~(1 << PC4);					// clock:Low

	for(keta = 0;keta < 4; keta++){
		for(b = 0x80; b != 0; b >>= 1){
			if((n & b) == 0){				// out bit = Low
				PORTC &= ~(1 << PC5);
			}else{							// out bit = high
				PORTC |=  (1 << PC5);
			}

			if(keta == 3){					// Ō̂Sڂ̎
				if(b == 1){					// WԖڂ̃rbg̎
					PORTC |= (1 << PC2);	// LE:High
				}
			}
			nop();
			PORTC |=  (1 << PC4);			// clock=high
			nop();
			PORTC &=  ~(1 << PC4);			// clock=low
			PORTC &= ~(1 << PC2);			// LE=Low
		}
	}
}
// *****************************
//  邳̃Zbg
//     n:Configuration Code
//           0--255(D0:CM,D1:HC,D2--D7:CC0--CC5)
// *****************************
void	set_brightness(unsigned char n)
{
	set_special_mode();			//  Special [hɃZbg
	write_config(n);			//  Configuration Code ̏
	set_normal_mode();			//  Normal [hɃZbg

	set_7seg_enable();			//  VZO\̗L̃Zbg
}

// *****************************
//  VZO\̖邳ݒ肷
//  @
//@@@@F邳iOFԈÂAPF炢AQFԖ邢j
//
// *****************************
void	set_7seg_bright(unsigned char n)
{
	if(n == 0){
		seg7_bright = SEG7_BRIGHT_LOW;			// ԈÂ
	}else if(n == 1){
		seg7_bright = SEG7_BRIGHT_MID;			// 炢
	}else{
		seg7_bright = SEG7_BRIGHT_HIGH;			// Ԗ邢
	}

	set_brightness(seg7_bright);				//  邳̃Zbg
}

